@@ -28,7 +28,7 @@ import subprocess
 
 import libcalamares
 
-from libcalamares.utils import check_target_env_call, check_target_env_output
+from libcalamares.utils import check_target_env_call
 
 import gettext
 
@@ -141,12 +141,18 @@ def get_kernel_params(uuid):
     use_systemd_naming = have_program_in_target("dracut") or (libcalamares.utils.target_env_call(["/usr/bin/grep", "-q", "^HOOKS.*systemd", "/etc/mkinitcpio.conf"]) == 0)
 
     partitions = libcalamares.globalstorage.value("partitions")
+    try:
+        gpu_drivers = libcalamares.globalstorage.value("gpuDrivers")
+    except KeyError:
+        pass
 
-    cryptdevice_params = []
     swap_uuid = ""
     swap_outer_mappername = None
     swap_outer_uuid = None
 
+
+    cryptdevice_params = []
+
     # Take over swap settings:
     #  - unencrypted swap partition sets swap_uuid
     #  - encrypted root sets cryptdevice_params
@@ -202,6 +208,9 @@ def get_kernel_params(uuid):
     if swap_outer_mappername:
         kernel_params.append(f"resume=/dev/mapper/{swap_outer_mappername}")
 
+    if "nvidia" in gpu_drivers:
+        kernel_params.append("nvidia_drm.modeset=1")
+
     return kernel_params
 
 
@@ -216,14 +225,6 @@ def create_systemd_boot_conf(installation_root_path, efi_dir, uuid, kernel, kern
     :param kernel_version: The kernel version string
     """
 
-    # Get the kernel params and write them to /etc/kernel/cmdline
-    # This file is used by kernel-install
-    kernel_params = " ".join(get_kernel_params(uuid))
-    kernel_cmdline_path = os.path.join(installation_root_path, "etc", "kernel")
-    os.makedirs(kernel_cmdline_path, exist_ok=True)
-    with open(os.path.join(kernel_cmdline_path, "cmdline"), "w") as cmdline_file:
-        cmdline_file.write(kernel_params)
-
     libcalamares.utils.debug(f"Configuring kernel version {kernel_version}")
 
     # get the machine-id
@@ -389,7 +390,7 @@ def get_efi_suffix_generator(name):
     if name.count("${") > 1:
         raise ValueError("EFI ID {!r} contains multiple generators".format(name))
     import re
-    prefix, generator_name = re.match(r"(.*)\${([^}]*)}$", name).groups()
+    prefix, generator_name = re.match("(.*)\${([^}]*)}$", name).groups()
     if generator_name not in ("SERIAL", "RANDOM", "PHRASE"):
         raise ValueError("EFI suffix {!r} is unknown".format(generator_name))
 
@@ -546,6 +547,14 @@ def install_systemd_boot(efi_directory):
                      "--path={!s}".format(install_efi_directory),
                      "install"])
 
+    # Get the kernel params and write them to /etc/kernel/cmdline
+    # This file is used by kernel-install
+    kernel_params = " ".join(get_kernel_params(uuid))
+    kernel_cmdline_path = os.path.join(installation_root_path, "etc", "kernel")
+    os.makedirs(kernel_cmdline_path, exist_ok=True)
+    with open(os.path.join(kernel_cmdline_path, "cmdline"), "w") as cmdline_file:
+        cmdline_file.write(kernel_params)
+
     for (kernel, kernel_type, kernel_version) in get_kernels(installation_root_path):
         create_systemd_boot_conf(installation_root_path,
                                  efi_directory,
@@ -578,8 +587,6 @@ def get_grub_efi_parameters():
         return "arm64-efi", "grubaa64.efi", "bootaa64.efi"
     elif efi_bitness == "64" and cpu_type == "loongarch64":
         return "loongarch64-efi", "grubloongarch64.efi", "bootloongarch64.efi"
-    elif efi_bitness == "64" and cpu_type == "riscv64":
-        return "riscv64-efi", "grubriscv64.efi", "bootriscv64.efi"
     elif efi_bitness == "64":
         # If it's not ARM, must by AMD64
         return "x86_64-efi", "grubx64.efi", "bootx64.efi"
@@ -607,7 +614,7 @@ def run_grub_mkconfig(partitions, output_file):
         check_target_env_call([libcalamares.job.configuration["grubMkconfig"], "-o", output_file])
 
 
-def run_grub_install(fw_type, partitions, efi_directory, install_hybrid_grub):
+def run_grub_install(fw_type, partitions, efi_directory):
     """
     Runs grub-install in the target environment
 
@@ -618,6 +625,7 @@ def run_grub_install(fw_type, partitions, efi_directory, install_hybrid_grub):
     """
 
     is_zfs = any([is_zfs_root(partition) for partition in partitions])
+
     # zfs needs an environment variable set for grub
     if is_zfs:
         check_target_env_call(["sh", "-c", "echo ZPOOL_VDEV_NAME_PATH=1 >> /etc/environment"])
@@ -627,62 +635,44 @@ def run_grub_install(fw_type, partitions, efi_directory, install_hybrid_grub):
         efi_bootloader_id = efi_label(efi_directory)
         efi_target, efi_grub_file, efi_boot_file = get_grub_efi_parameters()
 
-        grubinstall_command = [
-            libcalamares.job.configuration["grubInstall"],
-            "--target=" + efi_target,
-            "--efi-directory=" + efi_directory,
-            "--bootloader-id=" + efi_bootloader_id,
-            "--force"]
-
         if is_zfs:
-            # Needs environment to be set for GRUB, so go via the shell
-            check_target_env_call(["sh", "-c", "ZPOOL_VDEV_NAME_PATH=1 " + " ".join(grubinstall_command)]
+            check_target_env_call(["sh", "-c", "ZPOOL_VDEV_NAME_PATH=1 " + libcalamares.job.configuration["grubInstall"]
+                                   + " --target=" + efi_target + " --efi-directory=" + efi_directory
+                                   + " --bootloader-id=" + efi_bootloader_id + " --force"])
         else:
-            check_target_env_call(grubinstall_command)
-
+            check_target_env_call([libcalamares.job.configuration["grubInstall"],
+                                   "--target=" + efi_target,
+                                   "--efi-directory=" + efi_directory,
+                                   "--bootloader-id=" + efi_bootloader_id,
+                                   "--force"])
     else:
-        if libcalamares.globalstorage.value("bootLoader") is None and install_hybrid_grub:
-            efi_install_path = libcalamares.globalstorage.value("efiSystemPartition")
-            if efi_install_path is None or efi_install_path == "":
-                efi_install_path = "/boot/efi"
-            find_esp_disk_command = f"lsblk -o PKNAME \"$(df --output=source '{efi_install_path}' | tail -n1)\""
-            boot_loader_install_path = check_target_env_output(["sh", "-c", find_esp_disk_command]).strip()
-            if not "\n" in boot_loader_install_path:
-                libcalamares.utils.warning(_("Cannot find the drive containing the EFI system partition!"))
-                return
-            boot_loader_install_path = "/dev/" + boot_loader_install_path.split("\n")[1]
-        else:
-            boot_loader = libcalamares.globalstorage.value("bootLoader")
-            boot_loader_install_path = boot_loader["installPath"]
-            if boot_loader_install_path is None:
-                return
-            
-        # boot_loader_install_path points to the physical disk to install GRUB
-        # to. It should start with "/dev/", and be at least as long as the
-        # string "/dev/sda".
-        if not boot_loader_install_path.startswith("/dev/") or len(boot_loader_install_path) < 8:
-            raise ValueError(f"boot_loader_install_path contains unexpected value '{boot_loader_install_path}'")
+        assert efi_directory is None
+        if libcalamares.globalstorage.value("bootLoader") is None:
+            return
+
+        boot_loader = libcalamares.globalstorage.value("bootLoader")
+        if boot_loader["installPath"] is None:
+            return
 
         if is_zfs:
             check_target_env_call(["sh", "-c", "ZPOOL_VDEV_NAME_PATH=1 "
                                    + libcalamares.job.configuration["grubInstall"]
                                    + " --target=i386-pc --recheck --force "
-                                   + boot_loader_install_path])
+                                   + boot_loader["installPath"]])
         else:
             check_target_env_call([libcalamares.job.configuration["grubInstall"],
                                    "--target=i386-pc",
                                    "--recheck",
                                    "--force",
-                                   boot_loader_install_path])
+                                   boot_loader["installPath"]])
 
 
-def install_grub(efi_directory, fw_type, install_hybrid_grub):
+def install_grub(efi_directory, fw_type):
     """
     Installs grub as bootloader, either in pc or efi mode.
 
     :param efi_directory:
     :param fw_type:
-    :param install_hybrid_grub:
     """
     # get the partition from global storage
     partitions = libcalamares.globalstorage.value("partitions")
@@ -690,12 +680,8 @@ def install_grub(efi_directory, fw_type, install_hybrid_grub):
         libcalamares.utils.warning(_("Failed to install grub, no partitions defined in global storage"))
         return
 
-    if fw_type != "bios" and fw_type != "efi":
-        raise ValueError("fw_type must be 'bios' or 'efi'")
-
-    if fw_type == "efi" or install_hybrid_grub:
+    if fw_type == "efi":
         libcalamares.utils.debug("Bootloader: grub (efi)")
-        libcalamares.utils.debug(f"install_hybrid_grub: {install_hybrid_grub}")
         installation_root_path = libcalamares.globalstorage.value("rootMountPoint")
         install_efi_directory = installation_root_path + efi_directory
 
@@ -706,7 +692,7 @@ def install_grub(efi_directory, fw_type, install_hybrid_grub):
 
         efi_target, efi_grub_file, efi_boot_file = get_grub_efi_parameters()
 
-        run_grub_install("efi", partitions, efi_directory, install_hybrid_grub)
+        run_grub_install(fw_type, partitions, efi_directory)
 
         # VFAT is weird, see issue CAL-385
         install_efi_directory_firmware = (vfat_correct_case(
@@ -735,9 +721,9 @@ def install_grub(efi_directory, fw_type, install_hybrid_grub):
             efi_file_target = os.path.join(install_efi_boot_directory, efi_boot_file)
 
             shutil.copy2(efi_file_source, efi_file_target)
-    if fw_type == "bios" or install_hybrid_grub:
+    else:
         libcalamares.utils.debug("Bootloader: grub (bios)")
-        run_grub_install("bios", partitions, efi_directory, install_hybrid_grub)
+        run_grub_install(fw_type, partitions, None)
 
     run_grub_mkconfig(partitions, libcalamares.job.configuration["grubCfg"])
 
@@ -867,7 +853,7 @@ def install_refind(efi_directory):
     update_refind_config(efi_directory, installation_root_path)
 
 
-def prepare_bootloader(fw_type, install_hybrid_grub):
+def prepare_bootloader(fw_type):
     """
     Prepares bootloader.
     Based on value 'efi_boot_loader', it either calls systemd-boot
@@ -891,8 +877,8 @@ def prepare_bootloader(fw_type, install_hybrid_grub):
         try:
             efi_boot_loader = libcalamares.job.configuration["efiBootLoader"]
         except KeyError:
-            if fw_type == "efi" or install_hybrid_grub:
-                libcalamares.utils.warning("Configuration missing both efiBootLoader and efiBootLoaderVar on an EFI-enabled  "
+            if fw_type == "efi":
+                libcalamares.utils.warning("Configuration missing both efiBootLoader and efiBootLoaderVar on an EFI "
                                            "system, bootloader not installed")
                 return
             else:
@@ -917,7 +903,7 @@ def prepare_bootloader(fw_type, install_hybrid_grub):
     elif efi_boot_loader == "refind" and fw_type == "efi":
         install_refind(efi_directory)
     elif efi_boot_loader == "grub" or fw_type != "efi":
-        install_grub(efi_directory, fw_type, install_hybrid_grub)
+        install_grub(efi_directory, fw_type)
     else:
         libcalamares.utils.debug("WARNING: the combination of "
                                  "boot-loader '{!s}' and firmware '{!s}' "
@@ -932,15 +918,8 @@ def run():
     """
 
     fw_type = libcalamares.globalstorage.value("firmwareType")
-    boot_loader = libcalamares.globalstorage.value("bootLoader")
 
-    install_hybrid_grub = libcalamares.job.configuration.get("installHybridGRUB", False)
-    efi_boot_loader = libcalamares.job.configuration.get("efiBootLoader", "")
-
-    if install_hybrid_grub == True and efi_boot_loader != "grub":
-        raise ValueError(f"efi_boot_loader '{efi_boot_loader}' is illegal when install_hybrid_grub is 'true'!")
-
-    if boot_loader is None and fw_type != "efi":
+    if libcalamares.globalstorage.value("bootLoader") is None and fw_type != "efi":
         libcalamares.utils.warning("Non-EFI system, and no bootloader is set.")
         return None
 
@@ -953,7 +932,7 @@ def run():
             return None
 
     try:
-        prepare_bootloader(fw_type, install_hybrid_grub)
+        prepare_bootloader(fw_type)
     except subprocess.CalledProcessError as e:
         libcalamares.utils.warning(str(e))
         libcalamares.utils.debug("stdout:" + str(e.stdout))
